/* Nos conectamos al usuario Barna para crear su modelo */

CONNECT userTarra/userTarra@TARRA;

CREATE TABLE OFICINAS_TARRAGONA (
	Codigo		VARCHAR2(4) NOT NULL,
	Nombre		VARCHAR2(20) NOT NULL,
	Localidad	VARCHAR2(20) NOT NULL,
	Sede		VARCHAR2(10) NOT NULL,
	Director	VARCHAR2(4) NULL,
CONSTRAINT pk_oficinas PRIMARY KEY (Codigo),
CHECK (Sede ='Tarragona')
);

CREATE TABLE CLIENTES_TARRAGONA (
	Codigo	VARCHAR2(7) NOT NULL,
	DNI		VARCHAR2(10) NOT NULL,
	Nombre	VARCHAR2(20)NOT NULL,
	Oficina	VARCHAR2(4) NOT NULL,
CONSTRAINT pk_clientes PRIMARY KEY (Codigo), 
CONSTRAINT fk_clientes_oficinas FOREIGN KEY (Oficina)
		REFERENCES OFICINAS_TARRAGONA
);
CREATE TABLE POLIZAS_TARRAGONA (
	Numero			VARCHAR2(7) NOT NULL,
	Tipo			VARCHAR2(10) NOT NULL,
	Fecha 			DATE NOT NULL,
	Coberturas 		VARCHAR2(2000) NOT NULL,
	Prima 			NUMBER(10,2) NOT NULL,
	Beneficiarios 	VARCHAR2(2000) NOT NULL,
	Oficina 		VARCHAR2(4) NOT NULL,
CONSTRAINT pk_polizas PRIMARY KEY (Numero), 
CONSTRAINT fk_polizas_oficinas FOREIGN KEY (Oficina)
		REFERENCES OFICINAS_TARRAGONA,
CHECK (Tipo IN ('Vida','Hogar','Salud','Automovil','Viaje'))
);
CREATE TABLE CUOTAS_TARRAGONA (
	Numero			VARCHAR2(7) NOT NULL,
	Fecha 			DATE NOT NULL,
	Importe			NUMBER(10,2) NOT NULL,
	FechaProximoPago	DATE NOT NULL,
CONSTRAINT pk_cuotas PRIMARY KEY (Numero,Fecha), 
CONSTRAINT fk_cuotas_polizas FOREIGN KEY (Numero)
		REFERENCES POLIZAS_TARRAGONA ON DELETE CASCADE,
CHECK (FechaProximoPago>=Fecha)
);
CREATE TABLE DATOS_EMP_TARRAGONA (
	Codigo			VARCHAR2(7) NOT NULL,
	DNI 			VARCHAR2(10) NOT NULL,
	Telefono		VARCHAR2(12) NOT NULL,
	FecIniEmp		DATE NOT NULL,
	FecIniOficina	DATE NOT NULL,
	Oficina 		VARCHAR2(4) NOT NULL,
CONSTRAINT pk_datos_emp PRIMARY KEY (Codigo), 
CONSTRAINT fk_datos_emp_oficinas FOREIGN KEY (Oficina)
		REFERENCES OFICINAS_TARRAGONA,
CHECK (FecIniOficina >= FecIniEmp)
);
ALTER TABLE OFICINAS_TARRAGONA ADD (
CONSTRAINT fk_oficinas_empleados FOREIGN KEY (Director)
		REFERENCES DATOS_EMP_TARRAGONA
);

CREATE TABLE TITULARES_TARRAGONA (
	Cliente			VARCHAR2(7) NOT NULL,
	Poliza 			VARCHAR2(7) NOT NULL,
CONSTRAINT pk_titulares PRIMARY KEY (Cliente,Poliza), 
CONSTRAINT fk_titulares_clientes FOREIGN KEY (Cliente)
		REFERENCES CLIENTES_TARRAGONA,
CONSTRAINT fk_titulares_Polizas FOREIGN KEY (Poliza)
		REFERENCES POLIZAS_TARRAGONA ON DELETE CASCADE
);

CREATE OR REPLACE TRIGGER TG_OFICINAS_TARRAGONA
BEFORE INSERT OR UPDATE OR DELETE ON OFICINAS_TARRAGONA
FOR EACH ROW
BEGIN
 IF INSERTING THEN
  INSERT INTO OFICINAS@TO_BARNA A VALUES
    (:New.Codigo,:New.Nombre,:New.Localidad,
     :New.Sede,:New.Director);
 END IF;
 IF UPDATING THEN
  UPDATE OFICINAS@TO_BARNA A SET
  Codigo=:New.Codigo, Nombre=:New.Nombre,
  Localidad=:New.Localidad,Sede=:New.Sede,
  Director=:New.Director 
  WHERE A.Codigo = :New.Codigo;
 END IF;
 IF DELETING THEN
  DELETE OFICINAS@TO_BARNA A
  WHERE A.Codigo = :Old.Codigo;
 END IF;
END;
/

CREATE OR REPLACE TRIGGER TG_DATOS_EMP_TARRAGONA
BEFORE INSERT OR UPDATE OR DELETE ON DATOS_EMP_TARRAGONA
FOR EACH ROW
BEGIN
 IF INSERTING THEN
  INSERT INTO DATOS_EMP@TO_BARNA A VALUES
  (:New.Codigo,:New.DNI,:New.Telefono,:New.FecIniEmp,
   :New.FecIniOficina, :New.Oficina);
 END IF;
 IF UPDATING THEN
  UPDATE DATOS_EMP@TO_BARNA A SET
  Codigo=:New.Codigo, DNI=:New.DNI, Telefono =:New.Telefono,
  FecIniEmp =:New.FecIniEmp,FecIniOficina=:New.FecIniOficina,
  Oficina =:New.Oficina
  WHERE A.Codigo= :New.Codigo;
 END IF;
 IF DELETING THEN
  DELETE DATOS_EMP@TO_BARNA A
  WHERE A.Codigo= :Old.Codigo;
 END IF;
END;
/
